/* ==== helpmain.c ==== */
/* The main() function for a standalone helpfile reader
   (the actual game reads helpfiles with a builtin reader
   rather than this one, which is meant as a utility */

#include <stdio.h>
#include <ncurses.h>
#include "help.h"

extern WINDOW *win;

int main(int argc, char *argv[])
{
    /* Set up curses */
    int i;
    if(argc!=2) {
       fprintf(stderr,"This is porroghelp, the standalone version of the game\n");
       fprintf(stderr,"Porrog's built-in helpfile browser.\n");
       fprintf(stderr," Usage:\n");
       fprintf(stderr,"  %s helpfile\n",argv[0]);
       fprintf(stderr," where helpfile may or may not have a directory specified,\n");
       fprintf(stderr," and may or may not specify the file extension.\n");
       fprintf(stderr," If the given file has a bad format or the wrong extension,\n");
       fprintf(stderr," an error page is shown instead.\n");
       exit(1);
    }
    if (initscr()==NULL) {
       fprintf(stderr,"Yarg! Porroghelp couldn't initialise the terminal display! Giving up.\n");
       exit(1);
    }
    /* could do start_color() too */
    start_color();
    if(has_colors()&&COLOR_PAIRS>8) {
      for(i=0;i<8;i++) {
         init_pair(i+1,i,0);
      }
    }
    notimeout(stdscr,1);
    noecho();
    cbreak();

    /* Set up any peculiar variables that are wanted */
    move(0,40-19);
    addstr("porroghelp standalone helpfile viewer");
    refresh();
    win=newwin(22,78,1,1);
    /* run help function */
    show_help(argv[1]);
    wrefresh(win);delwin(win);
    touchwin(stdscr);refresh();
    /* shut down curses, then exit */
    endwin();
    /* That's done now, yes? */ 
    exit(0); /* or return(0)?; */
}

/* --- end of helpmain --- */

