/* ==== potions.c ==== */
/* Function pointers for potion effects, plus potion shuffling function */


#include <stdlib.h>
#include "porrog_defs.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "player_t.h"
/* protos */
#include "dXrng.h"
#include "crash.h"
#include "item_p.h"
#include "player_p.h"
#include "race_p.h"
#include "monster.h"
#include "ui.h"


void quaff_main()
{
    item_t *choice,*bottle;
    char nstring[80];
    /* pick a potion to quaff */
    show_messages("Quaff what?");
    choice=prompt_for_inv_item(0);
    if(NULL==choice) {
       show_messages("Cancelled");
       return;
    }
    if(choice->Itemtype->Type!=IG_POTION) {
       show_messages("You can't drink that!");
       return;
    }
    item_basename(nstring,choice);/* should take item knowledge into account*/
    sprintf(strbuf,"You open the bottle of %s and drink it...",nstring);

    /* NOw THE EFFECT */
    (choice->Itemtype->Typedata.Potion.Effect)(PUSE_QUAFF,NULL);

    /* If not yet known, it is now. ALWAYS */
    choice->Itemtype->Typedata.Potion.Known=1;
    /* NOW, we want to reduce the quantity of "choice" potions by 1,
    and gain 1 empty bottle. But do we do this by hand, or can we get
    some function that would usefully decrement an item's quantity or
    split it? */
    if(choice->Number==1) {
        p_discard_item(choice); /* So we don't screw up things! */
        free_item(choice); /* So we don't leak */
    } else (choice->Number)--; /* simple, eh? */
    /* SHOULD have some function that *attempts* to give player an item,
    and has it drop on the floor otherwise. Should p_get_item() do this
    automatically? */
    /* For now, just assume it's possible. THIS IS BAD! FIXME! */
    bottle=new_item(get_itemtype_by_name("bottle",IG_TOOL),1);
    p_get_item(bottle); /* should deal with mergine items, yes? */
    show_messages("You keep the empty bottle. Assuming you're not dead. FIXME");
}

void shuffle_potions()
{
    char *shuffletmp; /* temp var for shuffling */
    int shuffleable=0; /* number of potions that we can shuffle */
    int fixed,known;
    itemtype_t *pswap1,*pswap2;/* 2 potion types picked to have their
     Dnames swapped. Must be swappable types. */
    int nswaps,i;
    int pick,pick2,count; /* temp vars for picking pswap1 and pswap2 */
    itemtype_t *scan;
    /* Remember: If Prerec, then potion has one of 2 fixed appearances.
    But then if Known is 1, the Dname is available for shuffling with
    others (after which, the potion's Dname is set to its own Name.*/

    for(scan=first_its[IG_POTION];scan!=NULL;scan=scan->Next) {
       fixed=scan->Typedata.Potion.Prerec;
       if(fixed) /* work out if already known */
          known=(roll_dX(d10set[0])<fixed);
          /* if fixed=10, known=1; if fixed=0, known=0; */
       else known=0;/* unfixed potions are all unknown */
       scan->Typedata.Potion.Known=known; /* We only set that here */
       if(fixed==0 || known)
          shuffleable++;
    }
    nswaps=shuffleable*1.5; /* extra to be more likely everythings changed */
    /* Now shuffle, nswaps times */
    for(i=0;i<nswaps;i++) {
     /* Now pick a number 0 to shufflable-1 ? */
     pick=(shuffleable*((roll_dX(d12set[0]))+(12*roll_dX(d12set[1])) ))/144;
     /* was adapting next bit from pick_room and pick_room_from_area...
     but they appear wrong?!? */
     for(pswap1=first_its[IG_POTION],count=0;/*Keep going*/;pswap1=pswap1->Next)
     {
        count+=(( 0==pswap1->Typedata.Potion.Prerec)
               || pswap1->Typedata.Potion.Known);
        if(count>pick)break; /* I think */
        /* If we tested this from the "test" section of the for loop,
        we'd end up with the wrong element... */
     }
     pick2=(shuffleable*((roll_dX(d12set[0]))+(12*roll_dX(d12set[1])) ))/144;
     if(pick2==pick) pick2=(pick2+1)%shuffleable;
     /* Pick a second number, same range, not same as the first. */
     for(pswap2=first_its[IG_POTION],count=0;;pswap2=pswap2->Next) {
        count+=(( 0==pswap2->Typedata.Potion.Prerec)
               || pswap2->Typedata.Potion.Known);
        if(count>pick2)break;
     }
     /* Exchange Dname of both potions.*/
     shuffletmp                    = pswap1->Typedata.Potion.Dname;
     pswap1->Typedata.Potion.Dname = pswap2->Typedata.Potion.Dname;
     pswap2->Typedata.Potion.Dname = shuffletmp;

    }
    /* After doing  nswaps  shuffles, next part of sequence */

/*  for(all truly fixed potions) { */
    for(scan=first_its[IG_POTION];scan!=NULL;scan=scan->Next) {
       if(scan->Typedata.Potion.Known)
         scan->Typedata.Potion.Dname=scan->Name;
         /* doesn't need any other description, you're so familiar with it */
    }
    /* End of function */
}

void ineffectual_potion_effect(potionuse_t usg,void *tgt) {
    switch(usg) {
       case PUSE_QUAFF: /* quaff */
          show_messages("Nothing happens. At all.");
          return;
       case PUSE_DIP: /* dip */
          show_messages("Nothing happens.");
          return;
       case PUSE_THROWM: /* throw */
          show_messages("The monster isn't even hurt!");
          /* We don't actually know if target was a monster,
          or a blank space... or even the player!! */
          return;
       case PUSE_APPLY: /* apply */
          show_messages("You use the potion in the only obvious way: you pour it on the floor, leaving you with an empty bottle");
          /* exercise intelligence, wisdom, experience??? */
          return;
       case PUSE_QUAFFM:
       case PUSE_THROWP:
          bugger(REPTYPE_ITEMS|REPTYPE_AI,LOGLEVEL_FOOBAR,"Monsters can't use potions yet!");
          crash(INCONSISTENT);
    }
    return;
}

void water_potion_effect(potionuse_t usg,void *tgt) {
    char nstring[80];
    switch(usg) {
       case PUSE_QUAFF: /* quaff */
          show_messages("Tastes refreshing");
          return;
       case PUSE_DIP: /* dip */
          item_basename(nstring,(item_t *)tgt);
          sprintf(strbuf,"The %s looks cleaner! And wetter!",nstring);
          show_messages(strbuf);
          return;
       case PUSE_THROWM: /* throw */
          sprintf(strbuf,"The %s is slightly hurt by the breaking bottle.",((monster_t *)tgt)->Mobtype->Name);
          m_hurt((monster_t *)tgt,1);
          return;
       case PUSE_APPLY: /* apply */
          bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Applied an unapplyable potion!");
          crash(OUT_OF_BOUNDS); /* NO other application */
       case PUSE_QUAFFM:
       case PUSE_THROWP:
          bugger(REPTYPE_ITEMS|REPTYPE_AI,LOGLEVEL_FOOBAR,"Monsters can't use potions yet!!");
          crash(INCONSISTENT);
    }
    return;
}

void ink_potion_effect(potionuse_t usg,void *tgt) {
    char nstring[80];
    switch(usg) {
       case PUSE_QUAFF: /* quaff */
          show_messages("BLECH! That probably wasn't a good idea...");
          p_hurt(2);
          return;
       case PUSE_DIP: /* dip */
          item_basename(nstring,(item_t *)tgt);
          sprintf(strbuf,"The %s is covered in ink stains!",nstring);
          show_messages(strbuf);
          return;
       case PUSE_THROWM: /* throw AT monster */
          sprintf(strbuf,"The %s is hurt by the broken bottle, and covered in ink!",((monster_t *)tgt)->Mobtype->Name);
          show_messages(strbuf);
          m_hurt((monster_t *)tgt,1);
          return;
       case PUSE_APPLY: /* apply */
          show_messages("Maybe you could fill a pen with it. But instead, you throw the ink on the floor because you're stupid and don't know what you're doing!");
          return;
       case PUSE_QUAFFM:
       case PUSE_THROWP:
          bugger(REPTYPE_ITEMS|REPTYPE_AI,LOGLEVEL_FOOBAR,"Monsters cant use potions yet!");
          crash(INCONSISTENT);
    }
    return;
}

void magic_potion_effect(potionuse_t usg,void *tgt)
{
    switch(usg) {
       case PUSE_QUAFF:
          show_messages("Something magical happens!");
          break;
       default:
          show_messages("I haven't done this yet!");
    }
    return;
}

void exp_potion_effect(potionuse_t usg,void *tgt)
{
    int l; /* for working out roughly 1/4 to 1/2 of exp req */
    switch(usg) {
       case PUSE_QUAFF:
          show_messages("You feel more experienced!");
          l=player.Var_stats.Level-1;
          if (l<1) l=1;
          if (l>25) l=25;
          p_gainexp(global_scale_expreq(l)-global_scale_expreq(l-1));
          break;
       default:
          show_messages("I've not done this yet!");
    }
    return;
}

void poison_potion_effect(potionuse_t usg,void *tgt)
{
    switch(usg) {
       case PUSE_QUAFF:
          show_messages("Ooo, that's poisonous!");
          p_hurt(7);
          break;
       default:
          show_messages("I haven't done this yet!");
    }
    return; 
}

void poison2_potion_effect(potionuse_t usg,void *tgt)
{
    switch(usg) {
       case PUSE_QUAFF:
          show_messages("Arrg! That's really poisonous!");
          p_hurt(15);
          break;
       default:
          show_messages("I haven't done this yet!");
    }
    return;
}

void healing_potion_effect(potionuse_t usg,void *tgt)
{
    int hp,mhp;
    switch(usg) {
       case PUSE_QUAFF:
          hp=player.Var_stats.Hp;
          mhp=player.Var_stats.MaxHp;
          show_messages("Ooo, this is good.");
          if(hp==mhp) {
             show_messages("You feel robust!");
             /* Could boost MHP, but I think that'd work badly.
             COULD exercise con?*/
             player.Training.Con++;
             /* OH. We haven't fully implemented this stuff yet*/
             /* Should have a racial stats max pointer thingy in
             player structure */
          } else {
             if(mhp-hp>5) {
                show_messages("You feel a bit better!");
                player.Var_stats.Hp+=5;
             } else {
                show_messages("You feel fully healed!");
                player.Var_stats.Hp=mhp;
             }
          }
          break;
       default:
          show_messages("I've not done this yet!");
    }
    return;
}

/* --- */
